@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 4256 2007-12-29 21:28:37Z sysjkf $
+#   $Id: ConfigDefs.pl 4414 2008-04-22 09:06:02Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -55,9 +55,11 @@ addenvto			= AddEnvelopeToHeader
 addmshmac			= AddWatermark
 AllowObjectTags			= AllowObjectCodebaseTags
 AllowExternal			= AllowExternalMessageBodies
+allowmultsigs			= AllowMultipleHTMLSignatures
 AllowPartial			= AllowPartialMessages
 allowpasszips			= AllowPasswordProtectedArchives
 AllowWebBugTags			= AllowWebBugs
+assumeisdir			= MissingMailArchiveIs
 attachimage			= AttachImageToSignature
 attachimagename			= SignatureImageFilename
 attachimageinternalname		= SignatureImageImgFilename
@@ -102,6 +104,7 @@ disinfectedreporttext		= disinfectedreport
 envfromheader			= EnvelopeFromHeader
 envtoheader			= EnvelopeToHeader
 findphishing			= FindPhishingFraud
+fprotd6port			= FpscandPort
 gsscanner			= UseCustomSpamScanner
 gstimeout			= CustomSpamScannerTimeout
 gstimeoutlen			= CustomSpamScannertimeouthistory
@@ -125,12 +128,14 @@ inlinehtmlwarning		= inlinehtmlwarning
 inlinetextsig			= inlinetextsignature
 inlinetextwarning		= inlinetextwarning
 inqueuedir			= incomingqueuedir
+ipverheader			= ipprotocolversionheader
 keepspamarchiveclean		= keepspamandmcparchiveclean
 lastafterbatch			= alwayslookeduplastafterbatch
 lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
 loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
+logsaactions			= logspamassassinruleactions
 logsock				= syslogsockettype
 lookforuu			= finduuencodedfiles
 maxattachmentsize		= maximumattachmentsize
@@ -218,6 +223,7 @@ mcpdetail			= detailedmcpreport
 mcplistsascores			= includescoresinmcpreport
 mcpreqspamassassinscore		= mcprequiredspamassassinscore
 mcpwhitelist			= isdefinitelynotmcp
+syntaxcheck			= automaticsyntaxcheck
 unscannedheader			= unscannedheadervalue
 usedefaultswithmanyrecips       = usedefaultruleswithmultiplerecipients
 tagphishingsubject		= phishingmodifysubject
@@ -255,6 +261,7 @@ debug			0	no	0	yes	1
 debugspamassassin	0	no	0	yes	1
 deliverinbackground	1	no	0	yes	1
 lognonspam		0	no	0	yes	1
+logsaactions		0	no	0	yes	1
 logsilentviruses	0	no	0	yes	1
 logspam			0	no	0	yes	1
 logspeed		0	no	0	yes	1
@@ -265,6 +272,7 @@ showscanner		1	no	0	yes	1
 spamassassinautowhitelist 1	no	0	yes	1
 spliteximspool		0	no	0	yes	1
 storeentireasdfqf	0	no	0	yes	1
+syntaxcheck		1	no	0	yes	1
 usedefaultswithmanyrecips	0	no	0	yes	1
 #virusbeforespammcp	0	no	0	yes	1
 
@@ -295,6 +303,7 @@ clamavmaxratio                  250
 ClamdPort 3310
 CriticalQueueSize		800
 FileTimeout			20
+fprotd6port			10200
 GSTimeout			20
 GSTimeoutLen			20
 GunzipTimeout			50
@@ -409,7 +418,9 @@ ListSAScores		1	no	0	yes	1
 LogHTMLTags		0	no	0	yes	1
 LogPermittedFilenames	0	no	0	yes	1
 LogPermittedFiletypes	0	no	0	yes	1
+LogPermittedFileMimetypes	0	no	0	yes	1
 LookForUU		0	no	0	yes	1
+MissingMailArchiveIs	1	file	0	directory	1
 MultipleHeaders		append	append	append	replace	replace	add	add
 NameModifySubject	start	no	0	yes	1	start	start	end	end
 NoticeFullHeaders	1	no	0	yes	1
@@ -517,6 +528,7 @@ HighSpamSubjectText		{Spam?}
 Hostname			the MailScanner
 InfoHeader			
 InfoValue			Please contact the ISP for more information
+IPVerHeader			
 LocalPostmaster			postmaster
 MailHeader			X-MailScanner:
 mshmac				Watermark-secret
@@ -551,10 +563,12 @@ AddEnvTo		0	no	0	yes	1
 addmshmac		1	no	0	yes	1
 AllowIFrameTags		convert	no	0	yes	1	disarm	convert
 AllowFormTags		convert	no	0	yes	1	disarm	convert
+allowmultsigs		0	no	0	yes	1
 AllowObjectTags		convert	no	0	yes	1	disarm	convert
 AllowScriptTags		convert	no	0	yes	1	disarm	convert
 AllowPassZips		0	no	0	yes	1
 AllowWebBugTags		convert	no	0	yes	1	disarm	convert
+assumeisdir		1	file	0	directory	1
 attachimage		0	no	0	yes	1
 attachimagetohtmlonly	1	no	0	yes	1
 BlockEncrypted		0	no	0	yes	1
@@ -609,7 +623,9 @@ SpamDomainList
 webbugblacklist
 webbugwhitelist
 allowfilenames
+denyfilemimetypes
 denyfiletypes
+allowfilemimetypes
 allowfiletypes
 denyfilenames
 saactions
