@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm,v 1.64.2.25 2004/05/26 14:44:52 jkf Exp $
+#   $Id: MessageBatch.pm 4417 2008-04-23 14:28:08Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,11 +40,13 @@ use strict 'refs';
 no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
+use Time::HiRes qw ( time );
+use POSIX;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.64.2.25 $, 10;
+$VERSION = substr q$Revision: 4417 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -69,15 +71,23 @@ my $initialised = 0;
 #       out where they will need archiving. Do the archiving once we
 #       are about to remove it from the inqueue.
 sub new {
-  my $type = shift;
-  my $this = {};
+  my $type   = shift;
+  my $lint   = shift;
+  my $OnlyID = shift; # A batch only containing this message ID, if specified
+  my $this   = {};
 
   #print STDERR "In new MessageBatch\n";
 
   MailScanner::Log::DieLog("Tried to create a MessageBatch without calling initglobals")
     unless $initialised;
 
-  $global::MS->{mta}->CreateBatch($this);
+  if ($lint eq 'lint') {
+    # Fake a batch containing the Eicar message
+    bless $this, $type;
+    $this->CreateEicarBatch();
+  } else {
+    $global::MS->{mta}->CreateBatch($this, $OnlyID);
+  }
 
   $this->{starttime} = time;
 
@@ -94,14 +104,27 @@ sub EndBatch {
   my $totaltime  = $now - $this->{starttime};
   my $speed;
 
+  #print STDERR "Started at " . $this->{starttime} . "\n";
+  #print STDERR "Finished at " . $now . "\n";
+  #print STDERR "Tool " . $totaltime . "\n";
+
   $this->{endtime} = $now;
   $this->{totaltime} = $totaltime;
-  $totaltime = 1 unless $totaltime > 0; # Minimum of 1 second
+  $totaltime = 1 unless $totaltime > 0.001; # Minimum of 1 m-second
   $speed = ($totalbytes*1.0) / ($totaltime*1.0) if $totaltime > 0;
   $this->{bytespersecond} = $speed;
-  MailScanner::Log::InfoLog("Batch completed at %d bytes per second (%d / %d)",
-                            $speed, $totalbytes, $now-$this->{starttime})
-    if MailScanner::Config::Value('logspeed');
+  $speed = 0 if $speed > 1_000_000 || $speed < 0;
+  if (MailScanner::Config::Value('logspeed')) {
+    MailScanner::Log::InfoLog("Batch completed at %d bytes per second (%d / %d)",
+                              $speed, $totalbytes, $now-$this->{starttime});
+
+    # Work out current size of batch.
+    my $msgs = $this->{messages};
+    my $msgcount = scalar(keys %$msgs);
+
+    MailScanner::Log::InfoLog("Batch (%d message%s) processed in %.2f seconds",
+                              $msgcount, ($msgcount==1?'':'s'), $totaltime);
+  }
 }
 
 # Start the timing for a section of the main code
@@ -110,12 +133,7 @@ sub EndBatch {
 sub StartTiming {
   my $this = shift;
   my($varprefix, $usertext) = @_;
-
-  my $command = "";
-  my $now = time;
-
-  $command = '$this->{' . $varprefix . '_starttime} = $now;';
-  eval $command;
+  $this->{$varprefix . '_starttime'} = time;
 }
 
 # Stop the timing for a section of the main code
@@ -124,31 +142,27 @@ sub StopTiming {
   my $this = shift;
   my($varprefix, $usertext) = @_;
 
-  my $command = "";
   my $now = time;
   my $totaltime;
   my $speed;
 
   # totaltime = now - starttime
-  $command = '$totaltime = $now - $this->{' . $varprefix . '_starttime};';
-  eval $command;
+  $totaltime = $now - $this->{$varprefix . '_starttime'};
 
   # endtime = now
-  $command = '$this->{' . $varprefix . '_endtime} = $now;';
-  eval $command;
+  $this->{$varprefix . '_endtime'} = $now;
   # totaltime = totaltime
-  $command = '$this->{' . $varprefix . '_totaltime} = $totaltime;';
-  eval $command;
+  $this->{$varprefix . '_totaltime'} = $totaltime;
   $totaltime = 1 unless $totaltime > 0; # Minimum of 1 second
   # speed = bytes / totaltime
   $speed = ($this->{totalbytes}*1.0) / ($totaltime*1.0) if $totaltime > 0;
+  $speed = 0 if $speed > 1_000_000 || $speed < 0;
   # bytespersecond = speed
-  $command = '$this->{' . $varprefix . '_bytespersecond} = $speed;';
-  eval $command;
+  $this->{$varprefix . '_bytespersecond'} = $speed;
 
   MailScanner::Log::InfoLog("%s completed at %d bytes per second",
                             $usertext, $speed)
-    if MailScanner::Config::Value('logspeed');
+    if MailScanner::Config::Value('logspeed') && $speed > 0;
 }
 
 # This must be called as a class method before new() is used
@@ -192,6 +206,12 @@ sub RemoveDeletedMessages {
 
   my($id, $message, @badentries);
 
+  my $deleteifnotdelivering = 0;
+  $deleteifnotdelivering = 1
+    if MailScanner::Config::IsSimpleValue('keepspamarchiveclean') &&
+       !MailScanner::Config::Value('keepspamarchiveclean');
+  #print STDERR "Deleteifnotdelivering = $deleteifnotdelivering\n";
+
   #print STDERR "About to remove deleted messages\n";
   while(($id, $message) = each %{$this->{messages}}) {
     #print STDERR "Looking at $id for deletion\n";
@@ -202,7 +222,9 @@ sub RemoveDeletedMessages {
     }
     #MailScanner::Log::InfoLog("RemoveDeletedMessages: Deleting message $id")
     #  if $message->{deleted};
-    $message->DeleteMessage() if $message->{deleted};
+    #print STDERR "Message->deleted = " . $message->{deleted} . "and dontdeliver = " . $message->{dontdeliver} . "\n";
+    $message->DeleteMessage() if $message->{deleted} ||
+      ($message->{dontdeliver} && $deleteifnotdelivering);
     #delete $this->{messages}{$id} if $this->{messages}{$id}->{deleted};
   }
   foreach $id (@badentries) {
@@ -224,21 +246,33 @@ sub SpamChecks {
   MailScanner::Log::InfoLog("Spam Checks: Starting")
     if MailScanner::Config::Value('logspam');
 
+  # If the cache contents have expired then clean it all up first
+  MailScanner::SA::CheckForCacheExpire();
+
   while(($id, $message) = each %{$this->{messages}}) {
+    next if !$message->{scanmail};
     next if $message->{deleted};
-    next unless MailScanner::Config::Value('spamchecks', $message);
+    next unless MailScanner::Config::Value('spamchecks', $message) =~ /1/;
  
     #print STDERR "Spam checks for $id\n";
+
+    # Tell SpamAssassin to apply rules to binary attachments
+    # This relies on a patch to Util.pm in SpamAssassin, fetch the patch from
+    # http://www.mailscanner.info/mcp.html#patches
+    $MailScanner::MCP::SA_Decode_Binaries = 0;
+    $MailScanner::MCP::SA_Decode_Binaries = 1
+      if MailScanner::Config::Value('sadecodebins', $message) =~ /1/;
+
     $counter += $message->IsSpam();
 
     if (!MailScanner::Config::Value('spamdetail', $message)) {
       $message->{spamreport} = MailScanner::Config::LanguageValue($message,
                                ($message->{isspam}?'spam':'notspam'));
     }
-    #print STDERR "Spam header = \"" . $message->{spamreport} . "\"\n";
   }
-  MailScanner::Log::InfoLog("Spam Checks: Found $counter spam messages")
+  MailScanner::Log::NoticeLog("Spam Checks: Found $counter spam messages")
     if $counter>0;
+  $MailScanner::MCP::SA_Decode_Binaries = 0; # Reset 'are we in MCP' flag
   #print STDERR "$counter messages were spam\n";
 }
 
@@ -254,8 +288,10 @@ sub HandleSpam {
 
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
+    #print STDERR "Deleted = " . $message->{deleted} . ", isspam = " . $message->{isspam} . ", scanmail = " . $message->{scanmail} . ", spamwhitelisted = " . $message->{spamwhitelisted} . "\n";
     next if  $message->{deleted} ||
             !$message->{isspam}  ||
+            !$message->{scanmail} ||
              $message->{spamwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -277,6 +313,7 @@ sub HandleHam {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
+           !$message->{scanmail} ||
             $message->{isspam};
 
     #print STDERR "Ham checks for $id\n";
@@ -286,6 +323,22 @@ sub HandleHam {
   #print STDERR "Finished handling ham\n\n";
 }
 
+# Reject messages that come from people we want to reject. Send nice report
+# instead.
+sub RejectMessages {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    # Skip deleted and non-spam messages
+    next if $message->{deleted};
+    #print STDERR "May reject message $id\n";
+    $message->RejectMessage()
+      if MailScanner::Config::Value('rejectmessage',$message) !~ /0/;
+  }
+}
+
 # Do all the MCP checks.
 # Must have removed deleted messages from the batch first.
 sub MCPChecks {
@@ -299,8 +352,12 @@ sub MCPChecks {
   MailScanner::Log::InfoLog("MCP Checks: Starting")
     if MailScanner::Config::Value('logmcp');
 
+  # Tell SpamAssassin to apply rules to binary attachments
+  $MailScanner::MCP::SA_Decode_Binaries = 1;
+
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
+    next if !$message->{scanmail};
     next unless MailScanner::Config::Value('mcpchecks', $message);
  
     #print STDERR "Spam checks for $id\n";
@@ -312,9 +369,12 @@ sub MCPChecks {
     }
     #print STDERR "Spam header = \"" . $message->{spamreport} . "\"\n";
   }
-  MailScanner::Log::InfoLog("MCP Checks: Found $counter MCP messages")
+  MailScanner::Log::NoticeLog("MCP Checks: Found $counter MCP messages")
     if $counter>0;
   #print STDERR "$counter messages were spam\n";
+
+  # Tell SpamAssassin to return to normal behaviour
+  $MailScanner::MCP::SA_Decode_Binaries = 0;
 }
 
 
@@ -330,7 +390,8 @@ sub HandleMCP {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if  $message->{deleted} ||
-            !$message->{ismcp}  ||
+            !$message->{ismcp}   ||
+            !$message->{scanmail} ||
              $message->{mcpwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -352,6 +413,7 @@ sub HandleNonMCP {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
+           !$message->{scanmail} ||
             $message->{ismcp};
 
     #print STDERR "Ham checks for $id\n";
@@ -371,7 +433,16 @@ sub DeliverUnscanned {
   my($OutQ, @messages, $id, $message);
 
   while(($id,$message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
+
+    # This is for mail we don't want to touch at all
+    if (!$message->{scanmail}) {
+      $OutQ = MailScanner::Config::Value('outqueuedir', $message);
+      $message->DeliverUntouched($OutQ);
+      $message->{deleted} = 1; # This marks it for purging from disk
+      push @messages, $message;
+      next;
+    }
 
     if (!$message->{scanme}) {
       #print STDERR "Delivering unscanned message $id\n";
@@ -406,17 +477,28 @@ sub DeliverUnscanned {
 # Parse all the messages and expand all the attachments
 sub Explode {
   my $this = shift;
+  my($Debug) = @_;
 
   my($key, $message);
 
+  # jjh 2004-03-12 reap as many as we can.
+  # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+  unless ($Debug) {
+    1 until waitpid(-1, WNOHANG) == -1;
+  }
+
   #print STDERR "About to explode messages\n";
+  #print STDERR "Ignore errors about failing to find EOCD signature\n";
   umask $global::MS->{work}->{fileumask};
   while(($key, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
     #print STDERR "About to explode message $key $message\n";
     $message->Explode();
     # jjh 2004-03-12 reap as many as we can.
-    1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+    # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+    unless ($Debug) {
+      1 until waitpid(-1, WNOHANG) == -1;
+    }
   }
   umask 0077;
 }
@@ -426,21 +508,20 @@ sub Explode {
 sub VirusScan {
   my $this = shift;
 
-
   # Call them with the scanning settings
   MailScanner::Log::InfoLog("Virus and Content Scanning: Starting");
   my $viruses = MailScanner::SweepViruses::ScanBatch($this, 'scan');
-  MailScanner::Log::InfoLog("Virus Scanning: Found %d viruses", $viruses+0)
+  MailScanner::Log::NoticeLog("Virus Scanning: Found %d viruses", $viruses+0)
     if defined $viruses && $viruses>0;
 
   #MailScanner::Log::InfoLog("Other Checks: Starting");
   my $others  = MailScanner::SweepOther::ScanBatch($this, 'scan');
-  MailScanner::Log::InfoLog("Other Checks: Found %d problems", $others+0)
+  MailScanner::Log::NoticeLog("Other Checks: Found %d problems", $others+0)
     if defined $others && $others>0;
 
   #MailScanner::Log::InfoLog("Content Checks: Starting");
   my $content  = MailScanner::SweepContent::ScanBatch($this, 'scan');
-  MailScanner::Log::InfoLog("Content Checks: Found %d problems", $content+0)
+  MailScanner::Log::NoticeLog("Content Checks: Found %d problems", $content+0)
     if defined $content && $content>0;
 }
 
@@ -486,6 +567,25 @@ sub ReportBadMessages {
 }
 
 
+# Remove any infected spam or mcp from their archives. We have saved
+# all the spam+mcp archive places we stored this message, so go delete
+# the dirs and all the files in each one.
+sub RemoveInfectedSpam {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    #print STDERR "Message is infected\n" if $message->{infected};
+    next unless $message->{infected};
+    next unless
+      MailScanner::Config::Value('keepspamarchiveclean', $message) =~ /1/;
+    #print STDERR "Deleting " . join(',',@{$message->{spamarchive}}) . "\n";
+    unlink @{$message->{spamarchive}}; # Wipe the spamarchive files
+    @{$this->{spamarchive}} = (); # Wipe the spamarchive array
+  }
+}
+
 # Set up the entity2file and entity2parent hashes in every message
 sub CreateEntitiesHelpers {
   my $this = shift;
@@ -519,7 +619,7 @@ sub PrintFilenames {
   my($fnames, @filenames);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    #next if $message->{deleted};
     #print STDERR "Message $id has filenames ";
     @filenames = keys %{$message->{file2entity}};
     #print STDERR join(", ", @filenames) . "\n";
@@ -559,12 +659,25 @@ sub Clean {
   my($id, $message);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     #print STDERR "\nCleaning message $id\n";
     $message->Clean();
   }
 }
 
+# Zip up all the attachments in the messages, to save space on the
+# mail servers.
+sub ZipAttachments {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    next if $message->{deleted} || $message->{dontdeliver};
+    #print STDERR "\nZipping attachments, message $id\n";
+    $message->ZipAttachments();
+  }
+}
 
 # Combine the virus and other reports and types for all the messages.
 # Might change this to do it at source later.
@@ -595,9 +708,27 @@ sub QuarantineInfections {
                 =~ /1/;
     #print STDERR "Quarantining infections for $id\n";
     $global::MS->{quar}->StoreInfections($message);
+    $message->{quarantinedinfections} = 1; # Stop it quarantining it twice
   }
 }
 
+# Store all the disarmed files in the quarantine if they want me to.
+# Quarantine decision has to be done on a per-message basis.
+sub QuarantineModifiedBody {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    next unless $message->{bodymodified};
+    next if $message->{quarantinedinfections}; # Optimisation
+    next if MailScanner::Config::Value ('quarantinemodifiedbody', $message) !~
+            /1/;
+
+    $global::MS->{quar}->StoreInfections($message);
+    MailScanner::Log::NoticeLog("Quarantining modified message for %s", $id);
+  }
+}
 
 # Sign all the messages that were clean with a tag line saying
 # (ideally) that MailScanner is wonderful :-)
@@ -607,7 +738,7 @@ sub SignUninfected {
   my($id, $message);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     $message->SignUninfected()
       if MailScanner::Config::Value('signcleanmessages', $message) &&
          !$message->{infected};
@@ -626,7 +757,7 @@ sub DeliverUninfected {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     #print STDERR "Possibly delivering uninfected message $id\n";
     next if $message->{infected};
     #print STDERR "Delivering uninfected message $id\n";
@@ -690,7 +821,8 @@ sub DeliverOrDeleteSilent {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if !$message->{silent} || $message->{noisy} || $message->{deleted};
+    next if !$message->{silent}  || $message->{noisy} ||
+             $message->{deleted} || $message->{dontdeliver};
     #MailScanner::Log::WarnLog("Deliversilent for %s is %s", $message->{id},
     #                MailScanner::Config::Value('deliversilent', $message));
     if (MailScanner::Config::Value('deliversilent', $message)) {
@@ -704,7 +836,7 @@ sub DeliverOrDeleteSilent {
 
   if (@messages) {
     MailScanner::Mail::TellAbout(@messages);
-    MailScanner::Log::InfoLog("Silent: Delivered %d messages containing " .
+    MailScanner::Log::NoticeLog("Silent: Delivered %d messages containing " .
                               "silent viruses", scalar(@messages));
   }
 }
@@ -718,7 +850,7 @@ sub DeliverCleaned {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     $message->DeliverCleaned();
     #print STDERR "Deleting cleaned message " . $message->{id} . "\n";
     # BUGFIX: JKF $message->{deleted} = 1;
@@ -727,7 +859,7 @@ sub DeliverCleaned {
 
   MailScanner::Mail::TellAbout(@messages);
 
-  MailScanner::Log::InfoLog("Cleaned: Delivered %d cleaned messages",
+  MailScanner::Log::NoticeLog("Cleaned: Delivered %d cleaned messages",
                             scalar(@messages)) if @messages;
 }
 
@@ -746,7 +878,7 @@ sub WarnSenders {
     #print STDERR "Looking to warn sender of $id\n";
     next unless $message->{infected};
     #print STDERR "Warning sender of $id who is " .  $message->{from} . "\n";
-    next unless MailScanner::Config::Value('warnsenders', $message);
+    next unless MailScanner::Config::Value('warnsenders', $message) =~ /1/;
     #print STDERR "2Warning sender of $id who is " .  $message->{from} . "\n";
 
     # Count up the number of reasons why we want to warn the sender.
@@ -762,15 +894,20 @@ sub WarnSenders {
                   $warnviruses;
     $reasons++ if $message->{nameinfected}  &&
                   MailScanner::Config::Value('warnnamesenders', $message);
+    $reasons++ if $message->{sizeinfected} &&
+                  MailScanner::Config::Value('warnsizesenders', $message);
     $reasons++ if $message->{otherinfected} &&
                   MailScanner::Config::Value('warnothersenders', $message);
+    # JKF 19/12/2007
+    # $reasons++ if $message->{passwordinfected} &&
+    #               MailScanner::Config::Value('warnpasswordsenders', $message);
     next if $reasons==0;
 
     $message->WarnSender();
     $counter++;
   }
 
-  MailScanner::Log::InfoLog("Sender Warnings: Delivered %d warnings to " .
+  MailScanner::Log::NoticeLog("Sender Warnings: Delivered %d warnings to " .
                             "virus senders", $counter) if $counter;
 }
 
@@ -785,20 +922,37 @@ sub WarnLocalPostmaster {
 
   my($id, $message, $counter);
   my(%notices, $notice, %headers, %signatures, $email);
-  my(@posties, $posties, $postie, $sig);
+  my(@posties, $posties, $postie, $sig, %reasons, $reasons, $thisreason);
 
   # Create all the email messages
   $counter = 0;
   while(($id, $message) = each %{$this->{messages}}) {
     next if !$message->{infected};
     next if $message->{deleted} && !$message->{stillwarn};
-    next unless MailScanner::Config::Value('sendnotices', $message);
+    next unless MailScanner::Config::Value('sendnotices', $message) =~ /1/;
     $posties = MailScanner::Config::Value('noticerecipient', $message);
     return if $posties =~ /^\s*$/; # Return if no opsties defined
     @posties = split(" ", $posties);
     foreach $postie (@posties) {
       $headers{$postie} = $message->CreatePostmasterHeaders($postie)
         unless $headers{$postie};
+
+      # Change the subject to include the problem types
+      %reasons = ();
+      $reasons = "";
+      $reasons{virusinfected}     = 1 if $message->{virusinfected};
+      $reasons{filenameinfected}  = 1 if $message->{nameinfected};
+      $reasons{otherinfected}     = 1 if $message->{otherinfected};
+      $reasons{sizeinfected}      = 1 if $message->{sizeinfected};
+      # JKF 19/12/2007 $reasons{passwordinfected}  = 1 if $message->{passwordinfected};
+      $reasons{passwordprotected} = 1 if $message->{passwordprotected};
+      foreach $thisreason (sort keys %reasons) {
+        $reasons .= " : " if $reasons ne "";
+        $reasons .= MailScanner::Config::LanguageValue($message, "notice" .
+                                                       $thisreason);
+      }
+      $headers{$postie} =~ s/\nSubject:.*?\n/\nSubject: $reasons\n/si;
+
       $notices{$postie} .= $message->CreatePostmasterNotice();
       unless ($signatures{$postie}) {
         $sig = MailScanner::Config::Value('noticesignature', $message);
@@ -810,15 +964,17 @@ sub WarnLocalPostmaster {
   }
   while(($postie,$notice) = each %notices) {
     $email = $headers{$postie} . "\n" .
-      MailScanner::Config::LanguageValue($message, 'noticeheading') . ":\n" .
-      $notices{$postie} . "\n" . $signatures{$postie} . "\n";
+      #MailScanner::Config::LanguageValue($message, 'noticeheading') . ":\n" .
+      #$notices{$postie} . "\n" . $signatures{$postie} . "\n";
+      MailScanner::Config::LanguageValue($message, 'noticeprefix') . ": " .
+      $reasons . "\n" . $notices{$postie} . "\n" . $signatures{$postie} . "\n";
 
     $global::MS->{mta}->SendMessageString(undef, $email, $postie) or
       MailScanner::Log::WarnLog("Could not notify postmaster from $postie, %s",
                                 $!);
   }
 
-  MailScanner::Log::InfoLog("Notices: Warned about %d messages", $counter)
+  MailScanner::Log::NoticeLog("Notices: Warned about %d messages", $counter)
     if $counter;
 }
 
@@ -838,6 +994,7 @@ sub DisinfectAndDeliver {
   # such as DoS attacks.
   while(($id, $message) = each %{$this->{messages}}) {
     if ($message->{deleted} ||
+        $message->{dontdeliver} ||
         $message->{cantparse} ||
         $message->{badtnef} ||
         $message->{nameinfected} ||
@@ -856,7 +1013,7 @@ sub DisinfectAndDeliver {
   # Nothing to do?
   return unless @interesting;
 
-  MailScanner::Log::InfoLog("Disinfection: Attempting to disinfect %d " .
+  MailScanner::Log::NoticeLog("Disinfection: Attempting to disinfect %d " .
                             "messages", scalar(@interesting));
 
   # Save the infection reports, they will be needed to compare
@@ -884,7 +1041,7 @@ sub DisinfectAndDeliver {
   # Now re-scan the batch to find revised virus reports
   my $viruses = MailScanner::SweepViruses::ScanBatch($this, 'rescan');
   #print STDERR "Revised scanning found $viruses viruses\n";
-  MailScanner::Log::InfoLog("Disinfection: Rescan found only %d viruses",
+  MailScanner::Log::NoticeLog("Disinfection: Rescan found only %d viruses",
                             $viruses+0);
 
   # Look through the original list of reports, finding reports that
@@ -907,7 +1064,7 @@ sub ArchiveToFilesystem {
     $log .= " " . $id if $DidAnything;
   }
 
-  MailScanner::Log::InfoLog("Saved archive copies of%s", $log) if $log;
+  MailScanner::Log::NoticeLog("Saved archive copies of%s", $log) if $log;
 }
 
 
@@ -919,7 +1076,7 @@ sub StripHTML {
 
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{needsstripping};
     $message->StripHTML();
   }
@@ -934,7 +1091,7 @@ sub DisarmHTML {
   while(($id, $message) = each %{$this->{messages}}) {
     #print STDERR "In MessageBatch, tags are " .
     #             $message->{tagstoconvert} . "\n";
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{tagstoconvert};
     $message->DisarmHTML();
   }
@@ -948,12 +1105,23 @@ sub Encapsulate {
 
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{needsencapsulating};
     $message->EncapsulateMessage();
   }
 }
 
+# Add the virus statistics to the SpamAssassin results cache
+# so we know to keep the cache record for much longer.
+sub AddVirusInfoToCache {
+  my $this = shift;
+
+  my($id, $message);
+  while(($id, $message) = each %{$this->{messages}}) {
+    MailScanner::SA::AddVirusStats($message);
+  }
+}
+
 
 # This simply evaluates a known configuration parameter for every message
 # in the batch, whether it has been deleted or not. This solely exists so
@@ -962,10 +1130,84 @@ sub Encapsulate {
 sub LastLookup {
   my $this = shift;
 
+  my $start = time;
+
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
     MailScanner::Config::Value('lastlookup', $message);
   }
+
+  unless (MailScanner::Config::IsSimpleValue('lastlookup') &&
+          !MailScanner::Config::Value('lastlookup')) {
+    MailScanner::Log::InfoLog("\"Always Looked Up Last\" took %.2f seconds",
+                              time-$start+0.0)
+      if MailScanner::Config::Value('logspeed') =~ /1/;
+  }
+
+  # Lookup one remaining value after end of batch
+  # Putting in $this is against the rules as it isn't a message,
+  # but it doesn't actually cause any problems and gives MailWatch
+  # a way of getting hold of the batch statistics.
+  MailScanner::Config::Value('lastafterbatch', $this);
+}
+
+sub CreateEicarBatch {
+  my $batch = shift;
+
+  #print STDERR "Creating EICAR batch\n";
+  my $headerfileumask = $global::MS->{work}->{fileumask};
+
+  $batch->{messages} = {};
+  $batch->{totalmessages} = 1;
+  umask $headerfileumask;
+  # Create and write the header file
+  # Message number = 1
+  # Path = irrelevant as we're not actually reading anything
+  # It's a fake that we simulate ==> 1
+  my $newmessage = MailScanner::Message->new(1, '/tmp', 1);
+  @{$newmessage->{headers}} = ();
+  @{$newmessage->{to}} = ();
+  @{$newmessage->{touser}} = ();
+  @{$newmessage->{todomain}} = ();
+  $newmessage->{size} = 200;
+  $newmessage->{from} = 'sender@example.com';
+  $newmessage->{fromuser} = 'sender';
+  $newmessage->{fromdomain} = 'example.com';
+  push @{$newmessage->{to}}, 'recip@example.com';
+  push @{$newmessage->{touser}}, 'recip';
+  push @{$newmessage->{todomain}}, 'example.com';
+  $newmessage->{clientip} = '10.1.1.1';
+  $newmessage->{subject} = 'Virus Scanner Test Message';
+  push @{$newmessage->{headers}}, ('From: <sender@example.com>',
+                                   'To: <recip@example.com>',
+                                   'Subject: Virus Scanner Test Message',
+                                   'Mime-Version: 1.0',
+                                   'Content-Type: application/octet-stream; name="eicar.com"',
+                                   'Content-Transfer-Encoding: base64',
+'Content-Disposition: attachment; filename="eicar.com"'
+  );
+
+  $newmessage->{virusinfected} = 0;
+  $newmessage->{stillwarn} = 0;
+  $newmessage->{scanmail} = 1;
+
+  # Write out the .header file
+  $newmessage->WriteHeaderFile();
+
+  # Create a file of the body
+  my $fh = new FileHandle(">/tmp/MSLint.body.$$");
+  print $fh <<EOFLint;
+WDVPIVAlQEFQWzRcUFpYNTQoUF4pN0NDKTd9JEVJQ0FSLVNUQU5EQVJELUFOVElWSVJVUy1URVNU
+LUZJTEUhJEgrSCo=
+EOFLint
+  $fh->close();
+  $newmessage->{store}->{dpath} = "/tmp/MSLint.body.$$";
+
+  # Add it to the batch
+  $batch->{messages}{"1"} = $newmessage;
+  $newmessage->NeedsScanning(1);
+
+  umask 0077;
 }
 
 1;
